<?php
// $Id: media_flickr.utilities.inc,v 1.1.2.2 2009/06/25 16:37:34 aaron Exp $

/**
 *  @file
 *  Utility functions for Media: Flickr.
 */

/**
 *  Based on the Photo ID of a Flickr image, this will return the URL to the
 *  image itself.
 *  @param $photo_code
 *    The Flickr photo code.
 *  @param $width
 *  @param $height
 *    We use this to guess the actual size provided by Flickr.
 *  @param $cached
 *    If TRUE, then we'll cache the remote URL if the attempt to save the file
 *    locally fails.
 */
function _media_flickr_photo_url($photo_code, $width = 0, $height = 0, $cached = TRUE) {
  static $saves = 0;
  if ($photo_code) {
    $size = media_flickr_guess_size($width, $height);
    // If we're storing our files locally, then grab the filepath now.
    if ($store = variable_get('media_flickr_store_local', FALSE)) {
      $filepath = db_result(db_query("SELECT f.filepath FROM {files} f INNER JOIN {media_flickr_sizes} m ON m.fid = f.fid AND m.code = '%s' AND m.size = %d", $photo_code, $size));
    }
    // If we don't have a filepath, it's either because we're not storing
    // them locally, or it hasn't been saved locally yet.
    if (!$filepath && $store && (($max_saves = variable_get('media_flickr_max_saves', 10)) && ($saves < $max_saves) || !$max_saves)) {
      // If we've attempted to save a thumbnail and failed,
      // let's not try again for awhile.
      if ($cached && $cache = cache_get('media_flickr:'. $photo_code .':'. $size, 'cache')) {
        return $cache->data;
      }
      else {
        // Let's attempt to store our file locally.
        $file = media_flickr_store_local($photo_code, $size);
        if ($file['filepath']) {
          // Success! Let's grab that path.
          $filepath = $file['filepath'];
        }
        else {
          // Oops - it didn't save, so let's just grab the remote URL.
          $filepath = media_flickr_photo_remote_url($photo_code, $size);
          // Let's also cache it for awhile, as this can take a long time.
          cache_set('media_flickr:'. $photo_code .':'. $size, url($filepath), 'cache', time() + variable_get('emfield_cache_duration', 3600));
        }

        // We should only attempt to save a few photos at a time,
        // to keep from initial slow page loads.
        $saves++;
      }
    }
    // We're not storing locally, or have otherwise failed,
    // so now we'll just grab the remote URL.
    if (!$filepath) {
      $filepath = media_flickr_photo_remote_url($photo_code, $size);
    }
    // URLize that path...
    $filepath = url($filepath, array('absolute' => TRUE));
    return $filepath;
  }
}

function _media_flickr_store_local($photo_code, $size) {
  // Obviously, only go forward if we have a photo_code.
  if ($photo_code) {
    // Get the URL to the original thumbnail.
    $thumbnail = media_flickr_photo_remote_url($photo_code, $size);

    // Go forward only if we have a URL to go by.
    if ($thumbnail) {
      // The new file will be associated with the global user.
      global $user;

      // Attempt to fetch the thumbnail from the provided URL.
      $image = file_get_contents($thumbnail);

      // Only go forward if we actually have an image stream.
      if ($image) {

        // Add in our check of the the file name length.
        $validators['file_validate_name_length'] = array();

        // Allow for transliteration, which will take unicode data and convert
        // it to US-ASCII for better file storage.
        if (module_exists('transliteration')) {
          // Transliterate our original URL.
          $thumbnail = transliteration_get($thumbnail);
        }

        // Our new filepath will be in the form of media-flickr-7351895427047.jpg.
        $basename = 'media-flickr-'. $photo_code .'-'. $size .'.'. pathinfo($thumbnail, PATHINFO_EXTENSION);

        // Get the base Drupal files path.
        $directory = file_directory_path();
        $dir = variable_get('media_flickr_image_path', '');
        if ($dir) {
          // Add the field's image path here.
          $directory .= '/'. $dir;
        }
        // Create a new filepath from our desired filename.
        $filepath = file_create_filename($basename, $directory);

        // Begin building file object.
        $file = new stdClass();
        $file->uid = $user->uid;
        // Strip out the query if provided.
        $file->filename = parse_url($basename, PHP_URL_PATH);
        $file->filepath = parse_url($filepath, PHP_URL_PATH);

        // If we have mimedetect, then do so. Otherwise we make a best guess
        // based on the filename.
        $file->filemime = module_exists('mimedetect') ? mimedetect_mime($file) : file_get_mimetype($file->filename);

        // Rename potentially executable files, to help prevent exploits.
        if (preg_match('/\.(php|pl|py|cgi|asp|js)$/i', $file->filename) && (substr($file->filename, -4) != '.txt')) {
          $file->filemime = 'text/plain';
          $file->filepath .= '.txt';
          $file->filename .= '.txt';
        }

        $file->source = 'media_flickr_fetch_remote_thumbnail';
        $file->destination = file_destination($file->filepath, $replace);
        $file->filesize = strlen($image);

        // Call the validation functions.
        $errors = array();
        foreach ($validators as $function => $args) {
          array_unshift($args, $file);
          $errors = array_merge($errors, call_user_func_array($function, $args));
        }

        // Check for validation errors.
        if (!empty($errors)) {
          $message = 'The remote Flickr image %name could not be saved.';
          $arguments = array('%name' => $file->filename);
          if (count($errors) > 1) {
            $message .= '<ul><li>'. implode('</li><li>', $errors) .'</li></ul>';
          }
          else {
            $message .= ' '. array_pop($errors);
          }
          watchdog('media_flickr', $message, $arguments, WATCHDOG_ERROR);
          return 0;
        }

        if (!file_save_data($image, $file->filepath, FILE_EXISTS_RENAME)) {
          watchdog('file', 'Upload error. Could not move file %file to destination %destination.', array('%file' => $file->filename, '%destination' => $file->destination));
          return 0;
        }
        // If we made it this far it's safe to record this file in the database.
        $file->status = FILE_STATUS_PERMANENT;
        $file->timestamp = time();
        drupal_write_record('files', $file);

        $flickr = new stdClass();
        $flickr->fid = $file->fid;
        $flickr->size = $size;
        $flickr->code = $photo_code;
        drupal_write_record('media_flickr_sizes', $flickr);

        // Let modules add additional properties to the yet barebone file object.
        // This uses the future hook_file from d7's API. Not sure if anything
        // actually uses this right now, but they might in the future.
        foreach (module_implements('file_insert') as $module) {
          $function =  $module .'_file_insert';
          $function($file);
        }

        return (array)$file;
      }
    }
  }
}
